import xarray as xr
import matplotlib.pyplot as plt
import os
import glob
import json
import iris 
import numpy as np
import iris.plot as iplt
import matplotlib.gridspec as gridspec

dict_eval_metrics = {}
eval_metrics = ["delta_days", "trend_diff", "mean_annual_idays", "75p_annual_idays", "90p_annual_idays", "max_annual_idays"]
aliases_dict = {}
threshold_eval_corr = {}
threshold_eval_MAE = {}
# for season in ["djf", "mam", "jja", "son"]:
for season in ["ann", "djf", "mam", "jja", "son"]:
    atom_dict_eval_metrics = {k: [] for k in eval_metrics if "ann" not in season}
    # import OBS
    obs_file = f"/scratch/b/b381992/evaluation_OBS/OBSta700-ta850_g500-g1000_{season}_-10.20.35.44_19590101-20221231_min-intr-points-5_sahara--9.29.18.30-roll.31.json"
    with open(obs_file,"r") as f:
        obs_json = json.load(f)
    obs_ta_thresh = iris.load_cube(f"/scratch/b/b381992/evaluation_OBS/avgtap_700-850_1959-2022_01-12_sahara--9.29.18.30-roll.31-thresholds.nc")[-365:]
    obs_zg_thresh = iris.load_cube(f"/scratch/b/b381992/evaluation_OBS/gh_500-1000_1959-2022_01-12_sahara--9.29.18.30-roll.31-thresholds.nc")[-365:]

    # import CMIP
    if season == "ann":
        ta_thresh_dict = {}
        zg_thresh_dict = {}
    else:
        threshold_eval_corr[season] = []
        threshold_eval_MAE[season] = []
    bounds = {"djf": [0,31+28+31+1], "mam": [31+28+31+1,30+31+30+31+28+31+1], "jja": [31+28+31+30+31+30+1,31+28+31+30+31+31+30+31+30+1], "son":[31+28+31+30+31+31+30+31+30+1,31+28+31+30+31+31+30+31+30+30+31+30+2]}
    aliases = []
    aliases_dict[season]=[]
    cmip_files = sorted(glob.glob(f"/scratch/b/b381992/esmvaltool_output/recipe_CMIP6_sahara_intrusions_levante_20250325_090644/work/sahara_mass_identification/intrusion_days/evaluation/*_{season}_*"))
    for cmip_file in cmip_files:
        with open(cmip_file, "r") as f:
            cmip_json = json.load(f)
        alias=cmip_file.split("/")[-1].split(f"_{season}_")[0]
        if len(alias.split("_")) == 1:
            dataset=alias
            ensemble="*"
        else:
            dataset=alias.split(f"_{season}_")[0].split("_")[0]
            ensemble=alias.split(f"_{season}_")[0].split("_")[-1]

        file_ta_thresh = glob.glob(f"/scratch/b/b381992/esmvaltool_output/recipe_CMIP6_sahara_intrusions_levante_20250325_090644/work/sahara_mass_identification/intrusion_days/threshold_CMIP6_{dataset}_day_historical-ssp245_{ensemble}_avgtap_*_1959-2022.nc")
        if len(file_ta_thresh) != 1:
            raise KeyError("eee")
        cmip_ta_thresh = iris.load_cube(file_ta_thresh[0])[-365:]
        file_zg_thresh = glob.glob(f"/scratch/b/b381992/esmvaltool_output/recipe_CMIP6_sahara_intrusions_levante_20250325_090644/work/sahara_mass_identification/intrusion_days/threshold_CMIP6_{dataset}_day_historical-ssp245_{ensemble}_gh_*_1959-2022.nc")
        if len(file_zg_thresh) != 1:
            raise KeyError("ee")
        cmip_zg_thresh = iris.load_cube(file_zg_thresh[0])[-365:]

        if season == "ann":
            ta_thresh_corr = np.corrcoef(obs_ta_thresh.data, cmip_ta_thresh.data)[0,1]
            ta_thresh_MAE = np.mean(np.abs(obs_ta_thresh.data-cmip_ta_thresh.data))
            zg_thresh_corr = np.corrcoef(obs_zg_thresh.data, cmip_zg_thresh.data)[0,1]
            zg_thresh_MAE = np.mean(np.abs(obs_zg_thresh.data-cmip_zg_thresh.data))

            # threshold_eval_corr[season].append([ta_thresh_corr, zg_thresh_corr])
            # threshold_eval_MAE[season].append([ta_thresh_MAE, zg_thresh_MAE])

            ta_thresh_dict[f"{dataset}_{ensemble}"] = cmip_ta_thresh
            zg_thresh_dict[f"{dataset}_{ensemble}"] = cmip_zg_thresh
        else:
            ta_thresh_corr = np.corrcoef(obs_ta_thresh[bounds[season][0]:bounds[season][1]].data, cmip_ta_thresh[bounds[season][0]:bounds[season][1]].data)[0,1]
            ta_thresh_MAE = np.mean(np.abs(obs_ta_thresh[bounds[season][0]:bounds[season][1]].data-cmip_ta_thresh[bounds[season][0]:bounds[season][1]].data))
            zg_thresh_corr = np.corrcoef(obs_zg_thresh[bounds[season][0]:bounds[season][1]].data, cmip_zg_thresh[bounds[season][0]:bounds[season][1]].data)[0,1]
            zg_thresh_MAE = np.mean(np.abs(obs_zg_thresh[bounds[season][0]:bounds[season][1]].data-cmip_zg_thresh[bounds[season][0]:bounds[season][1]].data))

            threshold_eval_corr[season].append([ta_thresh_corr, zg_thresh_corr])
            threshold_eval_MAE[season].append([ta_thresh_MAE, zg_thresh_MAE])
            for metric in atom_dict_eval_metrics:
                if metric == "delta_days":
                    atom_dict_eval_metrics[metric].append(cmip_json["ndays"]-obs_json["ndays"])
                if metric == "trend_diff":
                    atom_dict_eval_metrics[metric].append((cmip_json["trend"][0]-obs_json["trend"][0])/obs_json["trend"][0])
                if metric == "mean_annual_idays":
                    atom_dict_eval_metrics[metric].append(np.mean(np.array(obs_json["bxp"]),axis=1)-np.mean(np.array(cmip_json["bxp"]),axis=1))
                if metric == "max_annual_idays":
                    atom_dict_eval_metrics[metric].append(np.max(np.array(obs_json["bxp"]),axis=1)-np.max(np.array(cmip_json["bxp"]),axis=1))
                if metric == "75p_annual_idays":
                    atom_dict_eval_metrics[metric].append(np.percentile(np.array(obs_json["bxp"]),75,axis=1)-np.percentile(np.array(cmip_json["bxp"]),75,axis=1))
                if metric == "90p_annual_idays":
                    atom_dict_eval_metrics[metric].append(np.percentile(np.array(obs_json["bxp"]),95,axis=1)-np.percentile(np.array(cmip_json["bxp"]),95,axis=1))
            dict_eval_metrics[season]=atom_dict_eval_metrics
        aliases.append(alias)
        aliases_dict[season].append(alias)
    datasets=list(set([i.split("_")[0] for i in aliases]))
    colors = plt.cm.get_cmap('tab20', len(datasets))




for season in ["djf", "mam", "jja", "son"]:

    heatmap = []
    for row, alias in enumerate(aliases):
        column_values = []
        for col, metric in enumerate(dict_eval_metrics[season]):
            val = dict_eval_metrics[season][metric][row]
            if isinstance(val, np.ndarray) or isinstance(val, list):
                val=list(val)
            else:
                val=[val]
            column_values += val
        heatmap.append(column_values)
    heatmap = np.array(heatmap)
    def plot_heatmaps(hm,cm1=0,cm2=0,cm3=0,cm4=0):
        plt.figure(figsize=(10,15))
        gs = gridspec.GridSpec(1, 4, width_ratios=[0.1, 0.1, 0.1, 1])  
        ax = plt.subplot(gs[0])
        if cm1 == 0:
            cmax = np.max(np.abs(hm[:,0:1]))
        else:
            cmax=cm1
        c=plt.pcolormesh(hm[:,0:1],cmap='RdBu_r', vmin=-cmax, vmax=cmax)
        plt.colorbar(c,label="days")
        ax.set_yticks(ticks=np.arange(hm.shape[0]) + 0.5, labels=aliases)
        ax.set_xticks(ticks=np.arange(hm[:,0:1].shape[1]) + 0.5, labels=[r"$\Delta$ days" for _ in range(hm[:,0:1].shape[1])])

        ax = plt.subplot(gs[1])
        if cm2 == 0:
            cmax = np.max(np.abs(np.array(threshold_eval_MAE[season])[:,0]))
        else:
            cmax=cm2
        c=plt.pcolormesh(np.array([np.array(threshold_eval_MAE[season])[:,0]]).T,cmap='RdBu_r', vmin=-cmax, vmax=cmax)
        ax.set_yticks(ticks=np.arange(hm.shape[0]) + 0.5, labels=["" for _ in aliases])
        plt.colorbar(c,label="K")
        ax.set_xticks(ticks=np.arange(1) + 0.5, labels=["ta_thresholds_corr"], rotation=90)
        ax = plt.subplot(gs[2])
        if cm3 == 0:
            cmax = np.max(np.abs(np.array(threshold_eval_MAE[season])[:,1]))
        else:
            cmax=cm3
        c=plt.pcolormesh(np.array([np.array(threshold_eval_MAE[season])[:,1]]).T,cmap='RdBu_r', vmin=-cmax, vmax=cmax)
        ax.set_yticks(ticks=np.arange(hm.shape[0]) + 0.5, labels=["" for _ in aliases])
        plt.colorbar(c,label="m")
        ax.set_xticks(ticks=np.arange(1) + 0.5, labels=["zg_thresholds_corr"], rotation=90)

        ax = plt.subplot(gs[3])
        if cm4 == 0:
            cmax = np.max(np.abs(hm[:,1:]))
        else:
            cmax=cm4
        c=plt.pcolormesh(hm[:,1:],cmap='RdBu_r', vmin=-cmax, vmax=cmax)
        ax.set_yticks(ticks=np.arange(hm.shape[0]) + 0.5, labels=["" for _ in aliases])
        plt.colorbar(c,label="1")
        lablls = [i.upper() for i in season]+[season.upper()]
        tcks = [list(dict_eval_metrics[season].keys())[1:][0]]+[f"{i}_{list(dict_eval_metrics[season].keys())[1:][1]}"for i in lablls]+[f"{i}_{list(dict_eval_metrics[season].keys())[1:][2]}"for i in lablls]+[f"{i}_{list(dict_eval_metrics[season].keys())[1:][3]}"for i in lablls]+[f"{i}_{list(dict_eval_metrics[season].keys())[1:][4]}"for i in lablls]
        ax.set_xticks(ticks=np.arange(hm[:,1:].shape[1]) + 0.5, labels=tcks, rotation=90)
        plt.tight_layout()

    plot_heatmaps(heatmap)
    plt.savefig(f"/scratch/b/b381992/eval2_{season}.png", bbox_inches="tight")

    mean=np.mean(heatmap, axis=0,keepdims=True)
    std=np.std(heatmap, axis=0,keepdims=True)
    heatmap = (heatmap-mean)/std
    plot_heatmaps(heatmap,cm2=3, cm3=150,cm4=4)
    plt.savefig(f"/scratch/b/b381992/standard_eval2_{season}.png", bbox_inches="tight")


plt.figure(dpi=200)
last_dataset = None
new=0
for alias, ta_thresh in ta_thresh_dict.items():
    dataset=alias.split("_")[0]
    if dataset == last_dataset:
        iplt.plot(ta_thresh, alpha=0.4, c=colors(new))
    else:
        new+=1
        iplt.plot(ta_thresh, alpha=0.4, c=colors(new-1), label=dataset)
        last_dataset = dataset
iplt.plot(ta_thresh.copy(obs_ta_thresh.data), c="k")
plt.ylabel(r"$\overline{\theta}_{700-850}$ / K")
plt.xlabel("day of year")
plt.legend(fontsize="xx-small",loc="upper left")
plt.savefig("/scratch/b/b381992/ta_thresh.png", bbox_inches="tight")


plt.figure(dpi=200)
last_dataset = None
new=0
for alias, zg_thresh in zg_thresh_dict.items():
    dataset=alias.split("_")[0]
    if dataset == last_dataset:
        iplt.plot(zg_thresh, alpha=0.4, c=colors(new))
    else:
        new+=1
        iplt.plot(zg_thresh, alpha=0.4, c=colors(new-1), label=dataset)
        last_dataset = dataset
iplt.plot(zg_thresh.copy(obs_zg_thresh.data), c="k")
plt.ylabel(r"$\Delta GH_{500-1000}$ / m")
plt.xlabel("day of year")
plt.legend(fontsize="xx-small",loc="upper left")
plt.savefig("/scratch/b/b381992/zg_thresh.png", bbox_inches="tight")

print("fi")
    # comparisons

    # save to dict

# plotting stage